// -*- c++ -*-
/***************************************************************************
                          gamemanager.h  -  description
                             -------------------
    begin                : lun mar 04 15:20:00 CET 2002
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _GAMEMANAGER_H_
#define _GAMEMANAGER_H_

#include <string>
#include <map>
#include <set>
#include <vector>

#define XOR(a,b) (bool)((int)(a) ^ (int)(b))

#include "variables.h"
#include "creature.h"

class Level;
class Board;

class GUICommandLine;
class GUICreature;
class GUIBoard;

class GameManager {
  friend class Level;
  friend class LevelAction;
public:
  GameManager(GUICreature *guic, GUIBoard *guib,
	      GUICommandLine *guicmd, Turn t=MARINE_TURN);
  ~GameManager(void);
  
  Turn getTurn(void);
  int getTurnNumber();
  string getLevelMessage(void);
  void getLevelInfo(QString &title, QString &auth, QString &nar, QString &obj, 
		    QString &forc, QString &cond);
  void Init(void);
  void InitBlipCards(void);
  bool isInGame(void);
  int getCurrentMaxId(void);
  void NewCreature(Creature *crea, bool force=false);
  void deleteCreature(Object *crea);
  void deleteCreature(int caseId);
 
  string chooseAction(int x, int y);

  void beginTurn();
  void endTurn();

  bool isVisibleFromMarine(int caseId);
  void CheckNonVisibleBlip(void);
  void CheckOverwatchTarget(void);
  void CheckExit(void);

  void RevertBlip(Blip *crea, bool isVoluntary);
  void deleteGenestealerInEntryZone(void);
  
  Creature * getCreature(int cId);
  bool selectCreature(Creature *c);
  bool selectCreature(int caseId);
  Creature * getSelected(void);

  int Attack(Creature *crea);
  int Shoot(Creature *crea, int caseId);
  bool ThrowFireDice(void);
  void turnDefenser(Object *, Direction dir);

  Flame * addFlame(int caseId);
  bool deleteFlame(void);
  void checkGenestealerInFlame(void);

  int nearestMarine(int caseid);
  bool isEntryBlocked(int caseId);

  const map<int, Creature *> *getList(void);
  const list<int>* getExitedUnits(ObjectType type);
  void printInterface(void);
  void printMissionObjectives(void);
  void printAllCreaturesInfo(void);
  void printSelectedCreature(void);
  void printOnViewCreature(void);
  bool onViewIsSelectable(void);
  void setOnView(int caseId);
  void selectOnView(void);

  set<int> * getObjectiveCases(void);
private:
  GUICreature *guic;
  GUIBoard *guib;
  GUICommandLine *guicmd;

  bool inGame;
  Level *level;
  Turn turn;
  int nbTurns;

  map<ObjectType, list<int> > exitedUnits;

  Creature *selected;
  set<Object *> *onView;
  Object *selectableOnView;
  int previousView;
  bool listChanged;

  map<int, Creature *> liste;
  map<int, Flame *> flames;
  vector<int> blipCards;
  vector<int>::iterator currBlipCard;
  set<int> currentBlockedEntries;
  int maxCreaId;
  int cmdPts;
};

extern GameManager *man;

#endif
