/***************************************************************************
                          guicommandline.cpp  -  description
                             -------------------
    begin                : lun mar 04 14:59:00 CET 2002
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

#include <sstream>
#include <cstdio>
using namespace std;

#include "guicommandline.h"

#include "board.h"
#include "gamemanager.h"
#include "guicreature.h"
#include "guiboard.h"


GUICommandLine::GUICommandLine(GUIBoard *b, GUICreature *c)
  : guiboard(b), guicrea(c), state(GUI_WAITING)
{}

GUICommandLine::~GUICommandLine() {}

void GUICommandLine::setState(interfaceState s)
{ state=s; }

void GUICommandLine::getFiringCommand(const char * command)
{
  int xcoord, ycoord;
  int nb = sscanf(command, "%d %d",&xcoord, &ycoord);  
  if (nb==2) {
    set<Object *> *obj=board->getObject(xcoord+ycoord*board->Xmax);
    bool continueFire=true;
    for (set<Object *>::iterator it=obj->begin();it!=obj->end(); it++) {
      if ((*it)->isMarinePlayer())
	continueFire=false;
    }
    if (continueFire) {
      char cmd[255];
      sprintf(cmd, "fire %d %d", xcoord, ycoord);
      getAllCommands(cmd);
    }
    else 
      getAllCommands(command);
  }
  else
    getAllCommands(command);
}

void GUICommandLine::getShootingCommand(const char * command)
{
  int xcoord, ycoord;
  int nb = sscanf(command, "%d %d",&xcoord, &ycoord);  
  if (nb==2) {
    set<Object *> *obj=board->getObject(xcoord+ycoord*board->Xmax);
    bool continueFire=true;
    for (set<Object *>::iterator it=obj->begin();it!=obj->end(); it++) {
      if ((*it)->isMarinePlayer())
	continueFire=false;
    }
    if (continueFire) {
      char cmd[255];
      sprintf(cmd, "shoot %d %d", xcoord, ycoord);
      getAllCommands(cmd);
    }
    else
      getAllCommands(command);
  }
  else
    getAllCommands(command);
}

void GUICommandLine::getCommand(const char *command)
{
  if (!getMainCommands(command)) {
    switch(state) {
    case GUI_SHOOT:
      getShootingCommand(command);
      break;
    case GUI_FIRE:
      getFiringCommand(command);
      break;
    default:
      getAllCommands(command);
    }
  }
  DrawInterface();
}

bool GUICommandLine::getMainCommands(const char *command)
{
  char act[64];
  sscanf(command,"%s", act);
  if (strcmp(act,"exit")==0 || strcmp(act,"quit")==0) {
    cout << "Exit the game. Hope you enjoy the game !\n";
    endOfExecution();
  }
  else if (strcmp(act, "newgame")==0)
    commandNewGame(command);
  else if (act[0]=='v' || strcmp(act, "view")==0)
    Write(commandView(command));
  else if (strcmp(command, "")==0); // Do nothing
  else return false;
  return true;
} 

void GUICommandLine::getAllCommands(const char *command)
{
  // TODO : not sure it will eradicate the bug...
  cout << "Command given : [" << command << "].\n";

  if (strcmp(command,"")==0)
    return;
  string temp = "> ";
  temp += command;
  Write(temp, true);
  
  char act[64];
  sscanf(command,"%s", act);
  if ((act[0]=='s' && act[1]=='e') || strcmp(act,"select")==0)
    Write(commandSelect(command));
  else if (act[0]=='m' || strcmp(act,"move")==0)
    Write(commandMove(command));
  else if ((act[0]=='t' && act[1]=='u') || strcmp(act,"turn")==0)
    Write(commandTurn(command));
  else if ((act[0]=='e' && act[1]=='n' && act[2]=='t') 
	   || strcmp(act,"enter")==0)
    Write(commandEnter(command));
  else if (act[0]=='r' || strcmp(act, "revert")==0)
    Write(commandRevert(command));
  else if ((act[0]=='o' && act[1]=='p') || strcmp(act, "open")==0)
    Write(commandOpen(command));
  else if (act[0]=='c' || strcmp(act, "close")==0)
    Write(commandClose(command));
  else if (act[0]=='a' || strcmp(act, "attack")==0)
    Write(commandAttack(command));
  else if ((act[0]=='o' && act[1]=='v') || strcmp(act,"overwatch")==0)
    Write(commandOverwatch(command));
  else if ((act[0]=='s' && act[1]=='h') || strcmp(act, "shoot")==0)
    Write(commandShoot(command));
  else if ((act[0]=='f' && act[1]=='i') || strcmp(act, "fire")==0)
    Write(commandFire(command));
  else if ((act[0]=='s' && act[1]=='t' && act[2]=='o') 
	   || strcmp(act, "stop")==0)
    Write(commandStop(command));
  else if (act[0]=='u' || strcmp(act, "unjam")==0)
    Write(commandUnjam(command));
  else if (act[0]=='l' || strcmp(act, "load")==0)
    Write(commandLoad(command));
  else if ((act[0]=='e' && act[1]=='n' && act[2]=='d') 
	   || strcmp(act,"endturn")==0)
    Write(commandEndTurn(command));
  else if ((act[0]=='t' && act[1]=='a') || strcmp(act,"takeordrop")==0)
    Write(commandTakeOrDrop(command));
  else if (act[0]=='h' || strcmp(act, "help")==0)
    Write(commandHelp(command));
  else {
    int xcoord, ycoord;
    int nb = sscanf(command, "%d %d",&xcoord, &ycoord);  
    if (nb==2)
      man->setOnView(xcoord+ycoord*board->Xmax);
    else 
      Write("Cannot recognized the command. Type 'help' for list of "
	    "commands.\n");
  }
  
  previous = string(command);
  man->CheckNonVisibleBlip();
}

string GUICommandLine::getPreviousCommand(void) {return previous;}

void GUICommandLine::commandNewGame(const char *command)
{
  if (!man->isInGame())
    man->Init();
  else
    Write("You have already begun a game.\n"
	  "Please restart the application to run another one");
}

string GUICommandLine::commandSelect(const char *command)
{
  stringstream res;

  int creatureId;
  int ycoord;
  int resb = sscanf(command, "%*s %d %d", &creatureId, &ycoord);
  if (resb==2) {
    // The user try to select a square.
    if (man->selectCreature(creatureId+board->Xmax*ycoord)) {
      Creature *crea = man->getSelected();
      res << "Creature " << creatureId << " selected.";
      crea->printInfo(2);
    }
    else
      res << "You can't select a creature there.";
  }
  else {
    Creature *crea = man->getCreature(creatureId);
    if (crea != 0) {
      if (man->selectCreature(crea)) {
	res << "Creature " << creatureId << " selected.";
	crea->printInfo(2);
      }
      else {
	res << "Creature " << creatureId << " cannot be selected.";
      }
    }
    else {
      res << "The creature " << creatureId << " does not exist.";
    }
  }
  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandMove(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before moving";
  }
  else {
    char direction[20];
    sscanf(command, "%*s %s",direction);
    bool resb=crea->Move(getDirectionType(direction));
    if (resb) {
      man->CheckExit();
      man->CheckOverwatchTarget();
      res << "The creature has moved.";
    }
    else
      res << "The creature cannot move to that direction.";
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandTurn(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before moving";
  }
  else {
    char dir;
    sscanf(command, "%*s %c",&dir);
    Direction direction;
    switch (dir) {
    case 'n': direction=NORTH; break;
    case 'e': direction=EAST; break;
    case 's': direction=SOUTH; break;
    case 'w': direction=WEST; break;
    default: direction=NODIRECTION; break;
    }

    bool resb=crea->TurnOrientation(direction);

    if (resb) {
      man->CheckOverwatchTarget();
      res << "Creature has been turned to direction : " << 
	printDirection(direction) << ".";
    }
    else {
      res << "Movement is not possible.";
    }
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandView(const char *command)
{
  stringstream res;
  char what[20];
  int id=0;
  int Yid=0;
  sscanf(command, "%*s %s %d %d",what, &id, &Yid);
  if ((what[0]=='m' && what[1]=='a')|| strcmp(what,"map")==0)
    guiboard->printBoard();
  else if (what[0]=='s' || strcmp(what,"selected")==0) {
    Creature *crea = man->getSelected();
    if (crea==0)
      return "No creature selected.";
    crea->printInfo(0);
  }
  else if ((what[0]=='c' && what[1]=='r') || strcmp(what,"creature")==0) {
    Creature *crea = man->getCreature(id);
    if (crea != 0)
      crea->printInfo(0);
    else
      res << "Creature  " << id << " does not exist.";
  }
  else if ((what[0]=='c' && what[1]=='a') || strcmp(what,"case")==0) {
    set<Object *> *objs = board->getObject(id+board->Xmax*Yid);
    if (objs==0)
      res << "Square " << id << " " << Yid << " is a wall.";
    else if (objs->size()==0)
      res << "Square " << id << " " << Yid << " is an empty square.";
    else 
      for (set<Object *>::iterator it=objs->begin(); it!=objs->end(); it++)
	(*it)->printInfo(0);
  }
  else if (what[0]=='l' || strcmp(what,"list")==0) {
    man->printAllCreaturesInfo();
  }
  else if ((what[0]=='m' && what[1]=='i')|| strcmp(what,"missiondescription"))
    printLevelInfo();
  else 
    res << "command view not understood.";
  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandEnter(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0)
    res << "There is no selected creature. Please select one before moving.";
  else {
    bool resb=crea->Enter();
    
    if (resb) {
      res << "The creature has enter the board.";
      man->CheckOverwatchTarget();
    }
    else
      res << "The creature cannot enter the board.";
  }
  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandRevert(const char *command)
{
  stringstream res;
  
  Creature *crea = man->getSelected();
  if (crea == 0)
    res << "There is no selected creature.Please select one before moving.";
  else {
    bool resb=crea->Revert();
    if (resb) {
      res << "Blip has been reverted.";
      man->RevertBlip((Blip *) crea, true);
    }
    else
      res << "Creature cannot be reverted.";
  }
  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandEndTurn(const char *command)
{
  state=GUI_WAITING;
  man->endTurn();
  switch (man->getTurn()) {
  case MARINE_TURN:
    Write("This is the turn of the Marine player.");
    break;
  case GENESTEALER_TURN:
    Write("This is the turn of the Genestealer player.");
    break;
  case CAT_TURN:
    Write("This is the turn of the CAT module.");
    break;
  }

  return ("");
}

string GUICommandLine::commandOpen(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    bool resb = crea->OpenDoor();

    if (resb) {
      res << "Door open.";
      man->CheckOverwatchTarget();
    }
    else
      res << "Cannot open door.";
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandClose(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    bool resb = crea->CloseDoor();

    if (resb) {    
      res << "Door closed.";
    }
    else
      res << "Cannot close door.";
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandAttack(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    int resi = man->Attack(crea);
    switch (resi) {
    case 0:
      res << "Cannot attack.";
      break;
    case 1:
      res << "Defenser and attacker had equal score. Nothing happens.";
      break;
    case 2:
      res << "Defenser was better : you were killed.";
      break;
    case 3:
      res << "Defenser was better but attack was not frontal...";
      break;
    case 4:
      res << "Attack is successful, you killed the creature.";
      break;
    default:
      res << "Error during attack command.";
    }
    if (resi!=0)
      man->CheckOverwatchTarget();
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandShoot(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    int xcoord, ycoord;
    int nb = sscanf(command, "%*s %d %d",&xcoord, &ycoord);  
    if (nb<2) {
      res << "Please choose the target you want to shoot.";
      setState(GUI_SHOOT);
      return res.str();
    }
    int resi = man->Shoot(crea, xcoord+ycoord*board->Xmax);
    switch (resi) {
    case -1:
      res << "Cannot shoot.";
      break;
    case 0:
      res << "Shoot has failed. Nothing happens.";
      break;
    case 1:
      res << "Good Shot! Creature killed.";
      break;
    case 2:
      res << "Shoot perfect, two creatures were killed.";
      break;
    default:
      res << "Error during shoot command.";
    }
  }
  return res.str();
}

string GUICommandLine::commandOverwatch(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    bool resb = crea->Overwatch();

    if (resb)    
      res << "Marine is now on Overwatch mode.";
    else
      res << "Creature cannot be put on Overwatch mode.";
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandUnjam(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    bool resb = crea->Unjam();

    if (resb)    
      res << "Marine has cleared his jammed bolter.";
    else
      res << "Creature cannot clear his bolter.";
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandFire(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    int xcoord, ycoord;
    int nb = sscanf(command, "%*s %d %d",&xcoord, &ycoord);  
    if (nb<2) {
      res << "Please choose the case you want to flame.";
      setState(GUI_FIRE);
      return res.str();
    }
    bool resb = crea->Fire(xcoord+ycoord*board->Xmax);

    if (resb)    
      res << "Flamer Marine has fired.";
    else
      res << "Creature cannot use the flame-thrower.";
  }
  
  return res.str();
}

string GUICommandLine::commandStop(const char *command)
{
  state=GUI_WAITING;
  return("returning in normal mode.");
}

string GUICommandLine::commandLoad(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    bool resb = crea->Reload();

    if (resb)    
      res << "Flamer Marine has reload his flamer.";
    else
      res << "Creature cannot reload his flamer.";
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::commandTakeOrDrop(const char *command)
{
  stringstream res;

  Creature *crea = man->getSelected();
  if (crea == 0) {
    res << "There is no selected creature. Please select one before.";
  }
  else {
    if (crea->hasAnObject()) {
      crea->dropTakenObject();
      res << "Your creature has dropped the object.";
    }
    else {
      if (crea->takeObject())
	res << "Your marine has taken the object.";
      else
	res << "Creature cannot take any object.";
    }
  }

  state=GUI_WAITING;
  return res.str();
}

string GUICommandLine::printDirection (Direction d)
{
  state=GUI_WAITING;
  switch (d) {
  case NORTH:
    return ("NORTH");
  case SOUTH:
    return ("SOUTH");
  case WEST:
    return ("WEST");
  case EAST:
    return ("EAST");
  default:
    return "";
  }
}

FullDirection GUICommandLine::getDirectionType(char *d)
{
  state=GUI_WAITING;
  if (strcmp(d,"north")==0 || strcmp(d,"n")==0)
    return NORTHNORTH;
  if (strcmp(d,"north-east")==0 || strcmp(d,"n-e")==0)
    return NORTHEAST;
  if (strcmp(d,"east")==0 || strcmp(d,"e")==0)
    return EASTEAST;
  if (strcmp(d,"south-east")==0 || strcmp(d,"s-e")==0)
    return SOUTHEAST;
  if (strcmp(d,"south")==0 || strcmp(d,"s")==0)
    return SOUTHSOUTH;
  if (strcmp(d,"south-west")==0 || strcmp(d,"s-w")==0)
    return SOUTHWEST;
  if (strcmp(d,"west")==0 || strcmp(d,"w")==0)
    return WESTWEST;
  if (strcmp(d,"north-west")==0 || strcmp(d,"n-w")==0)
    return NORTHWEST;
  return NORTHNORTH;
}

string GUICommandLine::commandHelp(const char *command)
{
  stringstream res;

  res << "List of commands :" << endl;
  res << " View List :     get list of all creatures.\n";
  res << " View MAp :      get an ASCII representation of the map.\n";
  res << " View Selected : get infos on current selected creature.\n";
  res << " View CReature <nb> : get infos on creature of id <nb>.\n";
  res << " View CAse <x> <y> : get infos on what is on square <x,y>.\n";
  res << " View MIssiondescription: get infos on current mission.\n\n";
  res << " SElect <nb> :   select creature <nb>.\n";
  res << " SElect <x> <y>: select creature in square <x,y>.\n";
  res << " Move <dir>:     move to direction dir (n|e|w|s|n-w|n-e|s-w|s-e).\n";
  res << " TUrn <north|east|south|west>.\n";
  res << " ENTer :         enter your selected creature from an entry zone into game.\n";
  res << " Revert :        revert your selected blip into genestealers.\n";
  res << " OPen :          open door.\n";
  res << " Close :         close door.\n\n";
  res << " Attack <dir>:   attack creature or door (specify dir only for blip).\n";
  res << " SHoot <x> <y>:  shoot at creature or door on square <x,y> (only for marine).\n";
  res << " FIre <x> <y>:   put a flame on square <x,y> (only for flamer).\n";
  res << " STOp:           stop firing or shooting mode.\n";
  res << " Load:           reload the flamer ammunitions.\n";
  res << " OVerwatch:      put your selected creature in a overwatch state.\n";
  res << " Unjam:          clear the jammed bolter of your selected marine.\n";
  res << " TAkeordrop:     take or drop object (for specific level).\n";
  res << " ENDturn:         change player turn.\n";
  res << "\n<dir> is one of (n,s,e,w).\n";
  res << "All commands can be simplified with their capitalised letters.";
  return res.str();
}
