/***************************************************************************
                          qcommandline.h  -  description
                             -------------------
    begin                : ven avr 19 13:20:00 CET 2002
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _QCOMMANDLINE_H_
#define _QCOMMANDLINE_H_

/*
  QCommandLine is the GUI interface for player interaction.
  This class takes all the input of the player and if it's a command execute it

*/

enum QState { Q_NONE, Q_DIRECTION, Q_LEVEL, Q_BOOLEAN, Q_ENDOFGAME, 
	      Q_LEVELINFO, Q_POPUPMSG };

#include <string>
#include <qthread.h>
using namespace std;

#include "guicommandline.h"

class QMultiLineEdit;
class QLineEdit;
class QLabel;
class QPushButton;
class QInterfaceImpl;
class QString;

class QCreature;
class QPixmapBoard;

class QCommandLine : public GUICommandLine, public QThread
{
public:
  QCommandLine(QPixmapBoard *qboard, QCreature *qcrea,
	       QInterfaceImpl *main);
  ~QCommandLine(void);
  
  virtual void run(void); // Qthread support.
  void endOfExecution(void);

  void getCommand(const char *command);
  void Write(string w, bool isCommand=false);

  void ResizeBoard(int x, int y);
  void DrawInterface(void);

  void GUIDrawInterface(void);
  void setObjectButtonsVisible(bool vis);
  void ClearListCreatures(void);
  void GUIClearListCreatures(void);
  
  Direction GetDirection(string txt);
  void GUI_GetDirection(void);
  QString GetLevelToPlay(void);
  void GUI_GetLevelToPlay(void);
  int GetCase(set<int> *cases, string txt);
  pair<int,Direction> GetCaseAndDirection(set<int> *cases, string txt);
  bool GetYesOrNo(string txt);
  void GUI_GetYesOrNo(void);
  void endOfGame(string txt);
  void GUI_EndOfGame(void);
  void printLevelInfo(QString &tit, QString &auth, QString &nar, QString &obj, 
		      QString &forc, QString &cond);
  void printLevelInfo(void);
  void GUI_PrintLevelInfo(void);
  void printPopupMessage(string msg);
  void GUI_PrintPopupMessage(void);

  void boardClick(int xcoord, int ycoord, bool isLeftButton);
  void GetEvent();

  void Game_Wait(void);
  void Game_Wake(void);

private:
  QInterfaceImpl *qmain;

  QMultiLineEdit *qinfo;
  QLineEdit *qcmd;
  QLabel *qview;
  QLabel *qsel;
  QMultiLineEdit *qlist;
  QPushButton *qViewButton;
  QLabel *qturninfo;

  QWaitCondition gameLock;
  QMutex gameMutex;

  bool guiRunning;
  string currCmd;

  QState qstate;
  string gui_msg;
  Direction gui_dir;
  int gui_caseId;
  QString gui_level;
  bool gui_res;
  QString leveltitle, levelauthor, levelnarration, levelobjectives,
    levelforces, levelcond;
};

#endif
