/***************************************************************************
                          xcommandline.cpp  -  description
                             -------------------
    begin                : lun mar 04 14:59:00 CET 2002
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

#include <cstdio>
#include <sstream>
#include <qstring.h>
using namespace std;

#include "xcommandline.h"

#include "gamemanager.h"

XCommandLine::XCommandLine(XBoard *x, XCreature *c)
  : GUICommandLine(x, c)
{}

XCommandLine::~XCommandLine(void)
{}

void XCommandLine::endOfExecution(void)
{ exit(0); }

void XCommandLine::MainLoop(void)
{
  char command[1024];
  while (1) {
    cout << "> " << flush;
    cin.getline(command, 1023);

    getCommand(command);
  }
}

void XCommandLine::Write(string w, bool isCommand)
{
  if (!isCommand)
    cout << w << endl;
}

void XCommandLine::printLevelInfo(QString &title, QString &auth, QString &nar,
				  QString &obj, QString &forc, QString &cond)
{
  cout << "Level title : " << title.latin1() << endl << endl;
  cout << "Author : " << auth.latin1() << endl << endl;
  cout << "Description :\n" << nar.latin1() << endl << endl;
  cout << "Objectives : " << obj.latin1() << endl << endl;
  cout << "Forces : " << forc.latin1() << endl << endl;
  cout << "Win conditions : " << cond.latin1() << endl << endl;
}

void XCommandLine::printLevelInfo(void)
{
  QString tit, auth, nar, obj, forc, cond;
  man->getLevelInfo(tit,auth,nar,obj,forc,cond);
  cout << "Level title : " << tit.latin1() << endl << endl;
  cout << "Author : " << auth.latin1() << endl << endl;
  cout << "Description :\n" << nar.latin1() << endl << endl;
  cout << "Objectives : " << obj.latin1() << endl << endl;
  cout << "Forces : " << forc.latin1() << endl << endl;
  cout << "Win conditions : " << cond.latin1() << endl << endl;
}

void XCommandLine::endOfGame(string msg)
{ cout << msg << endl; }

void XCommandLine::printPopupMessage(string msg) 
{ cout << msg << endl; }

void XCommandLine::DrawInterface(void) {}
void XCommandLine::ClearListCreatures(void) {}
void XCommandLine::setObjectButtonsVisible(bool vis) {}


Direction XCommandLine::GetDirection(string txt)
{
  char answer[512];
  cout << txt << endl << "Direction must be : n,s,e or w.\n";
  while(1) {
    cout << "? " << flush;
  
    cin.getline(answer, 511);
    char c='d';
    sscanf(answer,"%c",&c);
    switch (c) {
    case 'n':
      return NORTH;
    case 's':
      return SOUTH;
    case 'e':
      return EAST;
    case 'w':
      return WEST;
    default:
      cout << "Position is not good. Please try again.\n";
    }
  }
}

QString XCommandLine::GetLevelToPlay(void)
{
  char path[1024];
  cout << "Please give the level filename to play :\n";
  cout << "? " << flush;
  
  char fname[128];
  cin.getline(fname, 127);
  sprintf(path,"%s%clevels%c%s",DATA_DIR,DIR_DELIMITER,DIR_DELIMITER, fname);
  return QString(path);
}

pair<int,Direction> XCommandLine::GetCaseAndDirection(set<int> *cases, 
						      string txt)
{
  cout << txt << endl << _getStringCases(cases) << endl;
  char answer[512];
  
  Direction dir=NODIRECTION;
  int caseId=-1;
  bool contin=true;;
  while(contin) {
    contin=false;
    cout << "? " << flush;
  
    cin.getline(answer, 511);
    int xpos=-2, ypos=-2;
    char d='d';
    sscanf(answer,"%d %d %c",&xpos, &ypos, &d);
    if (d=='n') dir=NORTH;
    else if(d=='s') dir=SOUTH;
    else if(d=='e') dir=EAST;
    else if(d=='w') dir=WEST;
    else {
      contin=true;
      cout << "Direction is not good, please give square and direction again.\n";
    }

    caseId = xpos+ypos*board->Xmax;
    if (cases->find(caseId) == cases->end()) {
      contin=true;
      cout << "Position is not good. Please give a correct square.\n";
    }    
  }
  return pair<int,Direction>(caseId,dir);  
}

int XCommandLine::GetCase(set<int> *cases, string txt)
{
  cout << txt << endl << _getStringCases(cases) << endl;
  char answer[512];
  
  int caseId=-1;
  bool contin=true;;
  while(contin) {
    contin=false;
    cout << "? " << flush;
  
    cin.getline(answer, 511);
    int xpos=-2, ypos=-2;
    sscanf(answer,"%d %d",&xpos, &ypos);
    caseId = xpos+ypos*board->Xmax;
    if (cases->find(caseId) == cases->end()) {
      contin=true;
      cout << "Position is not good. Please give a correct square.\n";
    }    
  }
  return caseId;  
}

bool XCommandLine::GetYesOrNo(string txt)
{
  cout << txt;
  char answer[1024];
  cout << "? " << flush;
  
  cin.getline(answer, 1023);
  char b;
  sscanf(answer, "%c", &b);
  if (b=='y') return true;
  else return false;
}

string XCommandLine::_getStringCases(set<int> *lst, int chunk)
{
  stringstream res;
  int idx=0;
  for (set<int>::iterator it=lst->begin(); it!=lst->end(); it++) {
    res << " [" << (*it)%board->Xmax << " " << (*it)/board->Xmax << "] ";
    idx++;
    if (chunk && idx%chunk == 0)
      res << "\n";
  }
  return res.str();
}
